﻿// Originally developed by Andreas Guenther.

namespace TabsStudioAutoOpener
{
    public class AutoOpener : TabsStudioExt.ITabsStudioAddin, TabsStudioExt.IConfigurable
    {
        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte_)
        {
            engine = engine_;
            dte = dte_;
            isPreviewTabProperty = new TabsStudioExt.Util.IsPreviewTabProperty();

            storage = new OptionsStorage(engine.GetAddinSettingsDirectory());
            options = storage.Load();

            if (dte.Version.Equals("10.0"))
                windowsMonitor = new WindowsMonitor(engine);

            activateTimer = new System.Windows.Threading.DispatcherTimer();
            activateTimer.Interval = System.TimeSpan.FromMilliseconds(1);
            activateTimer.Tick += new System.EventHandler(ActivateTimerTick);

            openTimer = new System.Windows.Threading.DispatcherTimer();
            openTimer.Interval = System.TimeSpan.FromMilliseconds(1);
            openTimer.Tick += new System.EventHandler(OpenTimerTick);

            engine.TabCreated += OnTabCreated;

            solutionEventsSink = new SolutionEventsSink();
            System.IServiceProvider serviceProvider = new Microsoft.VisualStudio.Shell.ServiceProvider(
                    dte as Microsoft.VisualStudio.OLE.Interop.IServiceProvider);
            vsSolution = serviceProvider.GetService(typeof(Microsoft.VisualStudio.Shell.Interop.SVsSolution)) as
                Microsoft.VisualStudio.Shell.Interop.IVsSolution;
            vsSolution.AdviseSolutionEvents(solutionEventsSink, out sinkCookie);
        }

        public void OnDisconnection()
        {
            vsSolution.UnadviseSolutionEvents(sinkCookie);
            engine.TabCreated -= OnTabCreated;
            activateTimer.Stop();
            openTimer.Stop();

            if (windowsMonitor != null)
                windowsMonitor.Close();
        }

        public void Configure(System.Windows.Window parent)
        {
            OptionsDialog dialog = new OptionsDialog(options);
            dialog.Owner = parent;
            dialog.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterOwner;
            if (dialog.ShowDialog().GetValueOrDefault())
                storage.Save(options);
        }

        private void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            if (solutionEventsSink.IsSolutionOpening() || isPreviewTabProperty.IsPreviewTab(e.Tab.TabItem))
                return;

            if (windowsMonitor != null && windowsMonitor.IsOpened(e.Tab.TabExtensions[0].Window))
                return;

            tabToOpen = e.Tab;
            openTimer.Start();
        }

        private void OpenTimerTick(object sender, System.EventArgs e)
        {
            openTimer.Stop();
            if (tabToOpen == null)
                return;

            foreach (string file in engine.GetCorrespondingFiles(tabToOpen.TabItem))
                if (IsFileAllowed(file))
                    dte.ExecuteCommand("File.OpenFile", '"' + file + '"');

            for (int i = 0; i < tabToOpen.TabExtensions.Count; i++)
            {
                if (tabToOpen.TabExtensions[i].IsExtensionActive)
                {
                    if (tabToOpen.TabExtensions[i].Window != dte.ActiveWindow)
                    {
                        windowToActivate = tabToOpen.TabExtensions[i].Window;
                        activateTimer.Start();
                    }
                    break;
                }
            }

            tabToOpen = null;
        }

        private void ActivateTimerTick(object sender, System.EventArgs e)
        {
            activateTimer.Stop();
            if (windowToActivate != null)
            {
                windowToActivate.Activate();
                windowToActivate = null;
            }
        }

        private bool IsFileAllowed(string file)
        {
            string extension = GetExtension(System.IO.Path.GetFileName(file));
            if (extension.Length <= 1)
                return false;

            foreach (string allowedExtension in options.allowedExtensions.Split(','))
                if (string.Compare(extension, "." + allowedExtension, true) == 0)
                    return true;

            return false;
        }

        private string GetExtension(string fileName)
        {
            int firstDot = fileName.IndexOf('.');
            if (firstDot < 0)
                return "";
            return fileName.Substring(firstDot);
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private EnvDTE80.DTE2 dte;
        private OptionsStorage storage;
        private Options options;
        private WindowsMonitor windowsMonitor;
        private Microsoft.VisualStudio.Shell.Interop.IVsSolution vsSolution;
        private SolutionEventsSink solutionEventsSink;
        private uint sinkCookie;
        private System.Windows.Threading.DispatcherTimer activateTimer, openTimer;
        private EnvDTE.Window windowToActivate;
        private TabsStudioExt.Tab tabToOpen;
        private TabsStudioExt.Util.IsPreviewTabProperty isPreviewTabProperty;
    }
}
